#!/usr/bin/perl
# Return a "DEVICE" string (for inclusing in mdadm.conf) for exported/unexported
# devices. Two lines will always be outputed: 
#    EXPORTED=DEVICE /dev/XXX | NONE
#    UNEXPORTED=DEVICE /dev/sataX | NONE
#
# For example below we are using AOE to export /dev/sata[1-3] but /dev/sata4 is available
# to create a RAID on.
#    EXPORTED=DEVICE /dev/sata1 /dev/sata2 /dev/sata3
#    UNEXPORTED=DEVICE /dev/sata4  
#
# guilty-party: don_hiatt@pmc-sierra.com (Wed Nov  1 14:12:15 PST 2006)

my ($CONFIG_DIR) = "/etc/aoe";
my (%aoe);
my (%sata) = ("/dev/sata1" => "free", 
	      "/dev/sata2" => "free",
	      "/dev/sata3" => "free", 
	      "/dev/sata4" => "free");

# if we can't open the config dir then all devices are free
unless (opendir CONFDIR, $CONFIG_DIR) {
    print "EXPORTED=NONE\n";
    print_unexported(%sata);
    exit;
}

@confs= readdir(CONFDIR);
close (CONFDIR);
foreach $conf (@confs) {
    #next if $conf =~ /^\./;
    next if -d $conf; # skip directories
    my ($serial, $device, $block);
    my ($filename) = "$CONFIG_DIR/" . "$conf";
    #print "filename: $filename\n"; 
    open(CONFIG, $filename);
    while(<CONFIG>) {
	if (/^aoe_serial\s*=\s*(.*)/) {
	    $serial = $1;
	} elsif (/^aoe_device\s*=\s*(.*)/) {
	    $device = $1;
	} elsif (/^aoe_block_device\s*=\s*(.*)/) {
	    $block = $1;
	}
    }
    close(CONFIG);
    
    $aoe{$conf} = sprintf("%s-%02d %s", $serial, $device, $block);
    delete($sata{$block}); # remove from free sata hash
}

print "EXPORTED=";
if (keys %aoe) {
    print "DEVICE ";
    foreach $key (sort keys %aoe) {
	my ($device, $block) = split(/ /, $aoe{$key});
	print "$block ";
    }
    print "\n";
} else {
    print "NONE\n"
}

print_unexported(%sata);
exit; 

sub print_unexported {
    my (%hash) = @_;
    print "UNEXPORTED=";
    if (keys %hash) {
	print "DEVICE ";
	foreach $key (sort keys %hash) {
	    print "$key ";
	}
	print "\n";
    } else {
	print "NONE\n"
	}   
}
