#!/usr/bin/perl
# Scan /etc/services and return a random UDP/TCP port in the
# range [$min,$max] which is not already in use.
#
# With no command line options we scan for TCP ports in the
# range [$PORT_MIN, $PORT_MAX]
#
# guilty-party: don_hiatt@pmc-sierra.com (Fri Sep  8 17:03:34 PDT 2006)

my (%tcp_ports);
my ($filename) = "/etc/services";
my ($PROTOCOL) = "tcp";
my ($PORT_MAX) = 65534;
my ($PORT_MIN) = 1025;

# seed rand()
srand(time() ^ ($$ + ($$ << 15)));

#print "args($#ARGV): $ARGV[0], $ARGV[1], $ARGV[2]\n";
# You can also pass the protocl (e.g. tcp/udp) defaults to TCP
if ($#ARGV > -1 ) {
    if ($ARGV[0] eq "udp") {
	$PROTOCOL = "udp";
    } else {
	$PROTOCOL = "tcp";
    }
    #print "protocol: $PROTOCOL\n";
}

# You can also pass in the MIN/MAX on the command line
if ($#ARGV == 2) {
    #print "[min,max] = [$ARGV[1], $ARGV[2]]\n";
    if ($ARGV[1] < 1) {
	$PORT_MIN = 1;
    } else {
	$PORT_MIN = $ARGV[1];
    }

    if ($ARGV[2] > 65534) {
	$PORT_MAX = 65534;
    } else {
	$PORT_MAX = $ARGV[2];
    }
}

# generate a random number between [$min,$max]
sub getRand {
    my($min, $max) = @_;
    #print "[min,max] = [$max, $min]\n";
    $num = int (rand ($max-$min)) + $min;
    return $num;
}

unless (open SERVICES, $filename) {
    print STDERR "Can't open \"$filename\": $!\n";
    exit(1);
}

# iterate over /etc/services and build a hash of all TCP ports
while(<SERVICES>) {
    next if /^\#/;
    @service = split();
    next unless $service[1] =~ /$PROTOCOL/;
    $port = $service[1];
    $port =~ s/\/$PROTOCOL//g;
    next if ($port < $PORT_MIN);
    next if ($port > $PORT_MAX);
    $tcp_ports{$port} = $port;
    #print "$service[1] ($port)\n";
}
close( SERVICES );

# We need a random number *not* in the /etc/services
$rand_port = getRand($PORT_MAX, $PORT_MIN);
while ($tcp_ports{$rand_port} == $rand_port) {
    print "collision($rand_port)..trying again.\n";
    $rand_port = getRand($PORT_MAX, $PORT_MIN);
}
print "$rand_port\n";

exit;
